package com.fzm.common.config;

import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.properties.WxPayProperties;
import com.wechat.pay.contrib.apache.httpclient.WechatPayHttpClientBuilder;
import com.wechat.pay.contrib.apache.httpclient.auth.PrivateKeySigner;
import com.wechat.pay.contrib.apache.httpclient.auth.Verifier;
import com.wechat.pay.contrib.apache.httpclient.auth.WechatPay2Credentials;
import com.wechat.pay.contrib.apache.httpclient.auth.WechatPay2Validator;
import com.wechat.pay.contrib.apache.httpclient.cert.CertificatesManager;
import com.wechat.pay.contrib.apache.httpclient.exception.HttpCodeException;
import com.wechat.pay.contrib.apache.httpclient.exception.NotFoundException;
import com.wechat.pay.contrib.apache.httpclient.util.PemUtil;
import org.apache.http.impl.client.CloseableHttpClient;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.ClassPathResource;

import javax.annotation.Resource;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.security.GeneralSecurityException;
import java.security.PrivateKey;

/**
 * @author tangtuo
 * @date 2022/1/12 16:30
 */
@Configuration
public class WxPayConfig {

    @Resource
    private WxPayProperties wxPayProperties;


    @Bean
    public PrivateKey wxPrivateKey() {
        // 加载商户私钥（privateKey：私钥字符串地址）
        try {
            ClassPathResource resource = new ClassPathResource(wxPayProperties.getPrivateKeyPath());
            return PemUtil.loadPrivateKey(resource.getInputStream());
        } catch (IOException e) {
            throw GlobalException.newException(ResultCode.FAILED, "微信支付私钥加载失败 : " + e.getMessage());
        }
    }

    @Bean
    public CloseableHttpClient client(PrivateKey wxPrivateKey) throws NotFoundException, HttpCodeException, GeneralSecurityException, IOException {
        // 获取证书管理器实例
        CertificatesManager certificatesManager = CertificatesManager.getInstance();
        // 向证书管理器增加需要自动更新平台证书的商户信息
        String mchId = wxPayProperties.getMchId();
        String apiV3Key = wxPayProperties.getApiV3Key();
        String mchSerialNum = wxPayProperties.getMchSerialNum();
        certificatesManager.putMerchant(mchId, new WechatPay2Credentials(mchId,
                new PrivateKeySigner(mchSerialNum, wxPrivateKey)), apiV3Key.getBytes(StandardCharsets.UTF_8));
        // 从证书管理器中获取verifier
        Verifier verifier = certificatesManager.getVerifier(mchId);
        WechatPayHttpClientBuilder builder = WechatPayHttpClientBuilder.create()
                .withMerchant(mchId, mchSerialNum, wxPrivateKey)
                .withValidator(new WechatPay2Validator(verifier));
        // ... 接下来，你仍然可以通过builder设置各种参数，来配置你的HttpClient
        // 通过WechatPayHttpClientBuilder构造的HttpClient，会自动的处理签名和验签，并进行证书自动更新
        return builder.build();
    }
}
