package com.fzm.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.entity.Order;
import com.fzm.common.entity.dto.OrderDto;
import com.fzm.common.enums.OrderStatus;
import com.fzm.common.enums.PayScene;
import com.fzm.common.enums.PayType;
import com.fzm.common.mapper.OrderMapper;
import com.fzm.common.service.CopyrightApplyService;
import com.fzm.common.service.NftService;
import com.fzm.common.service.OrderService;
import com.fzm.common.utils.JwtUtil;
import com.fzm.common.utils.SnowflakeUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

/**
 * @author tangtuo
 * @date 2022/1/19 15:42
 */
@Service
@Transactional(rollbackFor = RuntimeException.class)
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements OrderService {

    @Resource
    private NftService nftService;

    @Resource
    private CopyrightApplyService copyrightApplyService;

    @Resource
    private SnowflakeUtil snowflakeUtil;

    @Resource
    private HttpServletRequest request;

    @Override
    public Order getByPaySceneAndProductId(Integer payScene, Integer productId) {
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pay_scene", payScene)
                .eq("product_id", productId);
        return this.getOne(queryWrapper);
    }

    @Override
    public void updateOrderStatus(Long out_trade_no, OrderStatus orderStatus) {
        Order order = new Order();
        order.setId(out_trade_no);
        order.setOrderStatus(orderStatus.getStatus());
        this.updateById(order);
    }

    @Override
    public Long createOrder(OrderDto orderDto) {
        Order order = new Order();
        long id = snowflakeUtil.snowflakeId();
        Integer productId = orderDto.getProductId();
        String orderName;
        if (PayScene.NFT.getCode().equals(orderDto.getPayScene())) {
            orderName = nftService.getById(productId).getName();
        } else {
            orderName = copyrightApplyService.getById(productId).getOpusName();
        }
        order.setId(id);
        order.setOrderName(orderName);
        order.setPayScene(orderDto.getPayScene());
        order.setOrderName(orderName);
        order.setFee(orderDto.getFee());
        order.setProductId(productId);
        order.setUserId(JwtUtil.getUserIdFromToken(request.getHeader("Authorization")));
        order.setOrderStatus(OrderStatus.PAYING.getStatus());
        this.save(order);
        return id;
    }

    @Override
    public Boolean updatePayType(Long orderId, Integer payType) {
        Order o = new Order();
        o.setId(orderId);
        o.setPayType(payType);
        return updateById(o);
    }
}
