package com.fzm.portal.listener;

import cn.fzm.chain.simplesdk.client.ParaChainClient;
import cn.fzm.chain.simplesdk.constant.TxStatusEnum;
import cn.fzm.chain.simplesdk.model.TxResult;
import com.fzm.common.constant.SystemConstant;
import com.fzm.common.entity.Nft;
import com.fzm.common.entity.User;
import com.fzm.common.entity.dto.NftPublishMsg;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.service.NftService;
import com.fzm.common.service.UserService;
import com.fzm.common.utils.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;

/**
 * @author tangtuo
 * @date 2022/1/21 16:49
 */
@Slf4j
@Component
public class NftListener {

    @Resource
    private ParaChainClient paraChainClient;

    @Resource
    private NftService nftService;

    @Resource
    UserService userService;

    @Resource
    private RedisUtil redisUtil;

    /**
     * 监听nft发行结果
     *
     * @param msg
     */
    @RabbitListener(queues = "nft.publish.queue")
    public void listenNftPublish(NftPublishMsg msg) {
        log.info("收到处理确认nft发行结果的消息: {}", msg);
        Nft nft = nftService.getById(msg.getId());
        User user = userService.getUserByWallet(nft.getPublishAddress());
        String hash = msg.getHash();
        // 确认交易结果
        TxResult txResult = paraChainClient.cycleConfirmTxWithHash(hash, true, 1000);
        if (!TxStatusEnum.SUCCESS.equals(txResult.getStatus())) {
            throw GlobalException.newException(ResultCode.PUBLISH_ERROR, txResult.getErrMsg().getValue());
        }
        String realHash = paraChainClient.getRealTxHashFromGrp(hash);
        /*TxResult txResult = paraChainClient.cycleConfirmTxWithHash(realHash, false, 1000);
        if (!TxStatusEnum.SUCCESS.equals(txResult.getStatus())) {
            throw GlobalException.newException(ResultCode.PUBLISH_ERROR, txResult.getErrMsg().getValue());
        }*/
        nft.setNftHash(realHash);
        nft.setTokenId(msg.getTokenId());
        nft.setPublishTime(new Date());
        nftService.updateById(nft);
        // 如果用户是第一次发行作品,把用户的isPublish修改成1,并清空用户统计的缓存信息
        if (SystemConstant.BOOLEAN_DATA_FALSE.equals(user.getIsPublish())) {
            User u = new User().setId(user.getId()).setIsPublish(SystemConstant.BOOLEAN_DATA_TRUE);
            userService.updateById(u);
            redisUtil.delete("user::statistic");
        }
    }
}
