package com.fzm.portal.listener;

import com.fzm.common.entity.dto.OrderProcessMsg;
import com.fzm.common.enums.CopyrightApplyState;
import com.fzm.common.enums.PayScene;
import com.fzm.common.service.CopyrightApplyService;
import com.fzm.common.service.NftService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author tangtuo
 * @date 2022/1/21 11:44
 */
@Slf4j
@Component
public class OrderListener {

    @Resource
    private NftService nftService;

    @Resource
    private CopyrightApplyService copyrightApplyService;

    @RabbitListener(queues = "order.process.queue")
    public void listenProcessOrder(OrderProcessMsg msg) {
        log.info("收到处理订单的消息: {}", msg);
        if (PayScene.NFT.getCode().equals(msg.getPayScene())) {
            // 如果支付场景是发行nft的话,需要把此订单的nft发行
            nftService.publish(msg.getProductId());
        } else {
            // 如果支付场景是版权申请的话,需要把当前订单对应的版权状态改为待核验
            copyrightApplyService.updateRegisterState(msg.getProductId(), CopyrightApplyState.TO_BE_REVIEWED.getCode());
        }
    }

}
